declare module "types-dominodb" {
  export interface DominoDB {
    useDocument: (option: UseAgentOptions) => Promise<any>;
    createDocument: (option: CreateDocumentOptions) => Promise<string>;
    bulkReadDocuments: (options: BulkReadOptions) => Promise<DominoResponse>;
    bulkReadAttachmentStream: (options: BulkReadOptions) => Promise<any>;
    bulkReadAttachmentStreamByUnid: (
      options: BulkReadAttachmentStreamByUnidOptions
    ) => Promise<any>;
    bulkReadRichTextStream: (
      options: BulkReadRichTextStreamOptions
    ) => Promise<any>;
    bulkReadRichTextStreamByUnid: (
      options: BulkReadRichTextStreamByUnidOptions
    ) => Promise<any>;
    bulkCreateAttachmentStream: (
      options: BulkCreateAttachmentStreamOptions
    ) => Promise<any>;
    bulkCreateRichTextStream: (
      options: BulkCreateAttachmentStreamOptions
    ) => Promise<any>;
    bulkReadDocumentsByUnid: (
      options: BulkReadByUnidOptions
    ) => Promise<DominoResponse>;
    bulkCreateDocuments: (
      options: BulkCreateOptions
    ) => Promise<DominoResponse>;
    bulkReplaceItemsByUnid: (
      options: BulkReplaceItemsByUnid
    ) => Promise<DominoResponse>;
    bulkReplaceItems: (options: BulkReplaceItems) => Promise<DominoResponse>;
    bulkReplaceDocumentsByUnid: (
      options: CreateDocumentOptions
    ) => Promise<DominoResponse>;
    bulkDeleteDocuments: (
      options: BulkDeleteDocumentsOptions
    ) => Promise<DominoResponse>;
    bulkDeleteDocumentsByUnid: (
      options: BulkDeleteDocumentsByUnidOptions
    ) => Promise<DominoResponse>;
    getFilePath: () => Promise<string>;
    getAcl: (options: GetACLOptions) => Promise<ACLResponse>;
    useAgent: (options: UseAgentOptions) => Promise<any>;
  }

  export interface GetACLOptions {
    accessToken: string;
  }

  export interface ACLResponse {
    aclEntryList: {
      name: string;
      permissions: any[];
      type: any;
      accessLevel: any;
      roleList: any[];
    }[];
    roleList: string[];
  }

  export interface DominoResponse {
    documents: any[];
    documentRange: {
      total: number;
      start: number;
      count: number;
    };
    errors: 0;
  }

  export interface UseDocumentOptions {
    unid: string;
  }

  export interface BulkCreateAttachmentStreamOptions {
    accessToken?: string;
  }

  export interface BulkReadAttachmentStreamOptions {
    query: string;
    queryLimits?: {
      maxViewEntriesScanned: number;
      maxDocumentsScanned: number;
      maxMilliSeconds: number;
    };
    accessToken?: string;
    start?: number;
    chunkSizeKb?: number;
    queryArgs?: any[];
    fileNames?: string[];
    computeOptions?: {
      computeWithForm: boolean;
    };
  }

  export interface BulkReadRichTextStreamOptions {
    query: string;
    queryLimits?: {
      maxViewEntriesScanned: number;
      maxDocumentsScanned: number;
      maxMilliSeconds: number;
    };
    accessToken?: string;
    chunkSizeKb?: number;
    queryArgs?: any[];
    fields?: string[];
  }

  export interface BulkReadRichTextStreamByUnidOptions {
    unids: string[];
    accessToken?: string;
    chunkSizeKb?: number;
    fields?: string[];
  }

  export interface BulkReadAttachmentStreamByUnidOptions {
    unids: string[];
    accessToken?: string;
    fileNames?: string[];
  }

  export interface CreateDocumentOptions {
    document: any;
    accessToken?: string;
    computeOptions?: {
      computeWithForm: boolean;
    };
    onErrorOptions?: onError;
    duplicateItems?: boolean;
  }

  export interface BulkCreateOptions {
    documents: any[];
    accessToken?: string;
    duplicateItems?: boolean;
    onErrorOptions?: onError;
  }

  export enum onError {
    CONTINUE= 'ON_ERROR_CONTINUE',
    ABORT_REMAINING= 'ON_ERROR_ABORT_REMAINING'
  }

  export interface BulkReadOptions {
    query: string;
    queryLimits?: {
      maxViewEntriesScanned: number;
      maxDocumentsScanned: number;
      maxMilliSeconds: number;
    };
    accessToken?: string;
    start?: number;
    queryArgs?: any[];
    itemNames?: string[];
    onErrorOptions?: onError;
    computeOptions?: {
      computeWithForm: boolean;
    };
  }

  export interface BulkReadByUnidOptions {
    unids: string[];
    accessToken?: string;
    itemNames?: string[];
  }

  export interface BulkDeleteDocumentsByUnidOptions {
    unids: string[];
    accessToken?: string;
  }

  export interface BulkDeleteDocumentsOptions {
    query: string;
    queryLimits?: {
      maxViewEntriesScanned: number;
      maxDocumentsScanned: number;
      maxMilliSeconds: number;
    };
    accessToken?: string;
    start?: number;
    count?: number;
    queryArgs?: any[];
    itemNames?: string[];
    computeOptions?: {
      computeWithForm: boolean;
    };
  }

  export interface BulkReplaceItemsByUnid {
    accessToken?: string;
    replaceItemsByUnid: any[];
  }

  export interface BulkReplaceItems {
    query: string;
    queryLimits?: {
      maxViewEntriesScanned: number;
      maxDocumentsScanned: number;
      maxMilliSeconds: number;
    };
    accessToken?: string;
    start?: number;
    count?: number;
    queryArgs?: any[];
    computeOptions?: {
      computeWithForm: boolean;
    };
    replaceItems: any;
  }

  export interface UseAgentOptions {
    name: string;
  }
}
